/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos
 * Encryption/decryption library. This library is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU Library General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: mcrypt.h,v 1.4 2000/02/08 12:14:54 nikos Exp $ */

#define MCRYPT_API_VERSION 19991117

#ifdef __cplusplus
extern "C" {
#endif

/* Definitions
 */
#define MCRYPT_UNKNOWN_ERROR -1
#define MCRYPT_ALGORITHM_MODE_INCOMPATIBILITY -2
#define MCRYPT_KEY_LEN_ERROR -3
#define MCRYPT_MEMORY_ALLOCATION_ERROR -4
#define MCRYPT_UNKNOWN_MODE -5
#define MCRYPT_UNKNOWN_ALGORITHM -6

/* generic - high level functions. 
 */
	    int mcrypt_module_open(char *algorithm,
					   char *a_directory, char *mode,
				   char *m_directory);
	    int mcrypt_module_close( int td);
/* returns thread descriptor */

	int mcrypt_generic_init(const int td, void *key, int lenofkey,
				void *IV);
	int mcrypt_generic_end(const int td);
	int mdecrypt_generic(int td, void *plaintext, int len);
	int mcrypt_generic(int td, void *plaintext, int len);

/* extra functions */

	int (mcrypt_self_test) (int);
	int (mcrypt_get_block_size) (int);
	int (mcrypt_get_iv_size) (int);
	int (mcrypt_get_key_size) (int);

	int (is_ok_algorithm) (int);	/* if this algorithm exists returns 0 */
	int (is_ok_mode) (int);	/* if this mode exists returns 0 */

/* If this is a block algorithm returns 1 
 */
	int (is_block_algorithm) (int);	

/* If the mode operates in blocks returns 1 
 */
	int (is_block_mode) (int);

/* If the mode is for block algorithms it returns 1 
 */
	int (is_block_algorithm_mode) (int td);
	int mcrypt_mode_has_iv ( int td);

/* Return a string containing the name of the algorithm/mode
 */
	char *(mcrypt_get_algorithms_name) (int td);
	char *(mcrypt_get_modes_name) (int td);

char** mcrypt_list_algorithms ( char* libdir, int* size);
char** mcrypt_list_modes ( char* libdir, int *size);

 /* Frees the memory allocated by the mcrypt_list_xxx() functions. 
  */
void mcrypt_free_p(char** p, int size);

 /* If mcrypt_xxx functions return an error code, and you supply this
  * to this function, it will print a human readable message
  */
void mcrypt_perror(int err);

 /* Self test for the specified algorithm 
  */
int mcrypt_module_self_test (char* algorithm, char* libdir);


 /* Checks the version of the specified module 
  */
int mcrypt_algorithm_version(char *algorithm, char *a_directory);
int mcrypt_mode_version(char *mode, char *a_directory);

#ifdef __cplusplus
}
#endif
