/* Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Library General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef LIBDEFS_H
#define LIBDEFS_H
#include <libdefs.h>
#endif
#include <bzero.h>
#include <mcrypt.h>
#include <xmemory.h>
#include <mcrypt_internal.h>

extern CRYPT_STREAM *streams;
extern int max_streams_to_date;
#if 0
extern int preloaded_symbols;
#endif

#ifdef HAVE_LIBPTHREAD
extern pthread_mutex_t counter_stream_lock;
extern pthread_mutex_t lt_mutex_lock;
#endif


int mcrypt_module_close(int td)
{
	LOCK;
	LT_LOCK;
	lt_dlexit ();
	lt_dlclose(streams[td].algorithm_handle);
	lt_dlclose(streams[td].mode_handle);
	LT_UNLOCK;
	
	streams[td].algorithm_handle = NULL;
	streams[td].mode_handle = NULL;
	UNLOCK;
	return 0;
}

#if 0
int mcrypt_modules_preopen() {

	preloaded_symbols=1;
	LTDL_SET_PRELOADED_SYMBOLS();
	
	return 0;
}
#endif 

int mcrypt_module_open(char *algorithm,
		       char *a_directory, char *mode, char *m_directory)
{
	CRYPT_STREAM *temp_streams;
	char *error;
	int i;
	int td;

	LOCK;
	/* Find a free stream to use */
	for (td = 0; td < max_streams_to_date; td++) {
		if (streams[td].algorithm_handle == NULL)
			break;
	}

	if (td >= max_streams_to_date) {
		MCRYPT_LOCK_ALL;	/* Locks all encryption/decryption threads */
		/* None available, so expand the streams list */
		temp_streams = mxrealloc(streams,
					 (max_streams_to_date +
					  1) * sizeof(CRYPT_STREAM));
		if (temp_streams == NULL) {
			MCRYPT_UNLOCK_ALL;
			UNLOCK;
			return MCRYPT_MEMORY_ALLOCATION_ERROR;
		}
		streams = temp_streams;
		Bzero(&(streams[max_streams_to_date]),
		      sizeof(CRYPT_STREAM));

		MCRYPT_UNLOCK_ALL;
		max_streams_to_date++;
	}
	UNLOCK;

	LOCK;
#ifdef HAVE_LIBPTHREAD
	pthread_mutex_init(&streams[td].mcrypt_lock, NULL);
#endif
	LT_LOCK;
	if (lt_dlinit()!=0) {
		LT_UNLOCK;
		UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}

	if (a_directory!=NULL) lt_dladdsearchdir(a_directory);
	if (m_directory!=NULL) lt_dladdsearchdir(m_directory); 
	lt_dladdsearchdir(LIBDIR);

/*	LTDL_SET_PRELOADED_SYMBOLS(); */

	streams[td].algorithm_handle = lt_dlopenext(algorithm);
	LT_UNLOCK;
	if (streams[td].algorithm_handle == NULL) {
		fputs(lt_dlerror(), stderr);
		fputs("\n", stderr);
		UNLOCK;
		return MCRYPT_UNKNOWN_ALGORITHM;
	}

	LT_LOCK;
	streams[td].mode_handle = lt_dlopenext(mode);
	LT_UNLOCK;
	if (streams[td].mode_handle == NULL) {
		fputs(lt_dlerror(), stderr);
		fputs("\n", stderr);
		LT_LOCK;
		lt_dlclose(streams[td].algorithm_handle);
		LT_UNLOCK;
		UNLOCK;
		return MCRYPT_UNKNOWN_MODE;
	}
	UNLOCK;

	if (is_block_algorithm_mode(td) != is_block_algorithm(td)) {
		mcrypt_module_close(td);
		return MCRYPT_ALGORITHM_MODE_INCOMPATIBILITY;
	}

	return td;
}



/* Modules' frontends */

int mcrypt_get_size(int td)
{
	int (*__mcrypt_get_size) (void);

	const char *error;
	char *s;

	s = streams[td].algorithm_handle;
	LT_LOCK;
	__mcrypt_get_size = lt_dlsym(s, "_mcrypt_get_size");
	LT_UNLOCK;
	if ((error = lt_dlerror())) {
		fputs(error, stderr);
		fputs("\n", stderr);
		return MCRYPT_UNKNOWN_ERROR;
	}
	return __mcrypt_get_size();
}

int mcrypt_mode_get_size(int td)
{
	int (*__mcrypt_get_size) (void);

	const char *error;
	char *s;

	s = streams[td].mode_handle;
	LT_LOCK;
	__mcrypt_get_size = lt_dlsym(s, "_mcrypt_mode_get_size");
	LT_UNLOCK;
	if ((error = lt_dlerror())) {
		fputs(error, stderr);
		fputs("\n", stderr);
		return MCRYPT_UNKNOWN_ERROR;
	}
	return __mcrypt_get_size();
}

void mcrypt_set_key(int td, void *a, void *b, int c, void *d, int e)
{
	void (*__mcrypt_set_key_stream) (void *, void *, int, void *, int);
	void (*__mcrypt_set_key_block) (void *, void *, int);
	const char *error;
	char *s;

	s = streams[td].algorithm_handle;
	if (is_block_algorithm(td) == 0) {
		/* stream */
		LT_LOCK;
		__mcrypt_set_key_stream = lt_dlsym(s, "_mcrypt_set_key");
		LT_UNLOCK;
		if ((error = lt_dlerror())) {
			fputs(error, stderr);
			fputs("\n", stderr);
			return;
		}
		__mcrypt_set_key_stream(a, b, c, d, e);
	} else {
		LT_LOCK;
		__mcrypt_set_key_block = lt_dlsym(s, "_mcrypt_set_key");
		LT_UNLOCK;
		if ((error = lt_dlerror())) {
			fputs(error, stderr);
			fputs("\n", stderr);
			return;
		}
		__mcrypt_set_key_block(a, b, c);
	}
}

int mcrypt_get_block_size(int td)
{
	int (*_mcrypt_get_block_size) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_block_size = lt_dlsym(s, "_mcrypt_get_block_size");
	LT_UNLOCK;
	return _mcrypt_get_block_size();
}

int __mcrypt_get_block_size(int td)
{
	int (*___mcrypt_get_block_size) (void);
	char *s;

	s = streams[td].algorithm_handle;

	LT_LOCK;
	___mcrypt_get_block_size = lt_dlsym(s, "_mcrypt_get_block_size");
	LT_UNLOCK;
	return ___mcrypt_get_block_size();
}


int mcrypt_get_iv_size(int td)
{
	int (*_mcrypt_get_iv_size) (int);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].mode_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_iv_size = lt_dlsym(s, "_mcrypt_get_iv_size");
	LT_UNLOCK;
	return _mcrypt_get_iv_size(td);
}

int mcrypt_get_key_size(int td)
{
	int (*_mcrypt_get_key_size) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_key_size = lt_dlsym(s, "_mcrypt_get_key_size");
	LT_UNLOCK;
	return _mcrypt_get_key_size();
}

int* mcrypt_get_supported_key_sizes(int td, int *len)
{
	int* (*_mcrypt_get_key_sizes) (int*);
	char *s;
	int *size;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_key_sizes = lt_dlsym(s, "_mcrypt_get_supported_key_sizes");
	LT_UNLOCK;
	size = _mcrypt_get_key_sizes(len);

	return size;
}

int is_block_algorithm(int td)
{
	int (*_is_block_algorithm) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_is_block_algorithm = lt_dlsym(s, "_is_block_algorithm");
	LT_UNLOCK;
	return _is_block_algorithm();
}

char *mcrypt_get_algorithms_name(int td)
{
	char *(*_mcrypt_get_algorithms_name) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_algorithms_name =
	    lt_dlsym(s, "_mcrypt_get_algorithms_name");
	LT_UNLOCK;
	return _mcrypt_get_algorithms_name();
}

void mcrypt_block_encrypt(int td, void *b)
{
	void (*__mcrypt_block_encrypt) (void *, void *);
	char *s;
	char *a;

	s = streams[td].algorithm_handle;
	a = streams[td].akey;
	LT_LOCK;
	__mcrypt_block_encrypt = lt_dlsym(s, "_mcrypt_encrypt");
	LT_UNLOCK;
	__mcrypt_block_encrypt(a, b);
}

void mcrypt_block_decrypt(int td, void *b)
{
	void (*__mcrypt_block_decrypt) (void *, void *);
	char *s;
	char *a;

	s = streams[td].algorithm_handle;
	a = streams[td].akey;
	LT_LOCK;
	__mcrypt_block_decrypt = lt_dlsym(s, "_mcrypt_decrypt");
	LT_UNLOCK;
	__mcrypt_block_decrypt(a, b);
}

void mcrypt_stream_encrypt(int td, void *b, int c)
{
	void (*__mcrypt_stream_encrypt) (void *, void *, int);
	char *s;
	char *a;

	s = streams[td].algorithm_handle;
	a = streams[td].akey;
	LT_LOCK;
	__mcrypt_stream_encrypt = lt_dlsym(s, "_mcrypt_encrypt");
	LT_UNLOCK;
	__mcrypt_stream_encrypt(a, b, c);
}

void mcrypt_stream_decrypt(int td, void *b, int c)
{
	void (*__mcrypt_stream_decrypt) (void *, void *, int);
	char *s;
	char *a;

	s = streams[td].algorithm_handle;
	a = streams[td].akey;
	LT_LOCK;
	__mcrypt_stream_decrypt = lt_dlsym(s, "_mcrypt_decrypt");
	LT_UNLOCK;
	__mcrypt_stream_decrypt(a, b, c);
}

int init_mcrypt(int td, void *buf, void *a, int b, void *c)
{
	int (*_init_mcrypt) (int, void *, void *, int, void *);
	char *s;

	s = streams[td].mode_handle;
	LT_LOCK;
	_init_mcrypt = lt_dlsym(s, "_init_mcrypt");
	LT_UNLOCK;
	return _init_mcrypt(td, buf, a, b, c);
}

int mcrypt(int td, void *buf, void *a, int b)
{
	int (*_mcrypt) (int, void *, void *, int);
	char *s;

	s = streams[td].mode_handle;
	LT_LOCK;
	_mcrypt = lt_dlsym(s, "_mcrypt");
	LT_UNLOCK;
	return _mcrypt(td, buf, a, b);
}

int mdecrypt(int td, void *buf, void *a, int b)
{
	int (*_mdecrypt) (int, void *, void *, int);
	char *s;

	s = streams[td].mode_handle;
	LT_LOCK;
	_mdecrypt = lt_dlsym(s, "_mdecrypt");
	LT_UNLOCK;
	return _mdecrypt(td, buf, a, b);
}

char *mcrypt_get_modes_name(int td)
{
	char *(*_mcrypt_get_modes_name) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].mode_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_mcrypt_get_modes_name = lt_dlsym(s, "_mcrypt_get_modes_name");
	LT_UNLOCK;
	return _mcrypt_get_modes_name();
}

int is_block_mode(int td)
{
	int (*_is_block_mode) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].mode_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_is_block_mode = lt_dlsym(s, "_is_block_mode");
	LT_UNLOCK;
	return _is_block_mode();
}

int mcrypt_mode_has_iv(int td)
{
	int (*__has_iv) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].mode_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	__has_iv = lt_dlsym(s, "_has_iv");
	LT_UNLOCK;
	return __has_iv();
}

int is_block_algorithm_mode(int td)
{
	int (*_is_a_block_mode) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].mode_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_is_a_block_mode = lt_dlsym(s, "_is_block_algorithm_mode");
	LT_UNLOCK;
	return _is_a_block_mode();
}

int mcrypt_self_test(int td)
{
	int (*_self_test) (void);
	char *s;

	MCRYPT_LOCK;
	s = streams[td].algorithm_handle;
	MCRYPT_UNLOCK;

	LT_LOCK;
	_self_test = lt_dlsym(s, "_mcrypt_self_test");
	LT_UNLOCK;
	
	return _self_test();
}

int mcrypt_module_self_test(char *algorithm, char *a_directory)
{
	char *error;
	int i;
	lt_dlhandle *_handle;
	int (*_self_test) (void);

	LT_LOCK;

	
	if (lt_dlinit()!=0) {
		LT_UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}
/*	LTDL_SET_PRELOADED_SYMBOLS(); */
	if (a_directory!=NULL) lt_dlsetsearchpath(a_directory);
	
	_handle = lt_dlopenext(algorithm);
	LT_UNLOCK;
	
	if (!_handle) {
		fputs(lt_dlerror(), stderr);
		fputs("\n", stderr);
		return MCRYPT_UNKNOWN_ERROR;
	}

	_self_test = lt_dlsym(_handle, "_mcrypt_self_test");

	i = _self_test();

	lt_dlclose(_handle);
	lt_dlexit();
	return i;
}

int mcrypt_algorithm_version(char *algorithm, char *a_directory)
{
	char *error;
	int i;
	lt_dlhandle _handle;
	int (*_version) (void);

	LT_LOCK;
	if (lt_dlinit()!=0) {
		LT_UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}
/*	LTDL_SET_PRELOADED_SYMBOLS(); */
	if (a_directory!=NULL) lt_dlsetsearchpath(a_directory);
	
	_handle = lt_dlopenext(algorithm);
	if (!_handle) {
		fputs(lt_dlerror(), stderr);
		fputs("\n", stderr);
		LT_UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}

	_version = lt_dlsym(_handle, "_mcrypt_algorithm_version");

	i = _version();

	lt_dlclose(_handle);
	lt_dlexit();
	LT_UNLOCK;
	
	return i;
}

int mcrypt_mode_version(char *mode, char *m_directory)
{
	char *error;
	int i;
	lt_dlhandle *_handle;
	int (*_version) (void);

	LT_LOCK;
	if (lt_dlinit()!=0) {
		LT_UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}
/*	LTDL_SET_PRELOADED_SYMBOLS(); */
	if (m_directory!=NULL) lt_dlsetsearchpath(m_directory); 
	
	_handle = lt_dlopenext(mode);
	if (!_handle) {
		fputs(lt_dlerror(), stderr);
		fputs("\n", stderr);
		LT_UNLOCK;
		return MCRYPT_UNKNOWN_ERROR;
	}

	_version = lt_dlsym(_handle, "_mcrypt_mode_version");

	i = _version();

	lt_dlclose(_handle);
	lt_dlexit();
	LT_UNLOCK;
	
	return i;
}

int mcrypt_get_algo_iv_size(int td)
{
    int (*_mcrypt_get_algo_iv_size) (void);
    char *s;

    MCRYPT_LOCK;
    s = streams[td].algorithm_handle;
    MCRYPT_UNLOCK;
    
    _mcrypt_get_algo_iv_size = lt_dlsym(s, "_mcrypt_get_algo_iv_size");
    return _mcrypt_get_algo_iv_size();
}
